__author__ = 'mnowotka'

# Order may be important

from chembl_core_model.models.general import *
from chembl_core_model.models.sourceInformation import *
from chembl_core_model.models.compounds import *
from chembl_core_model.models.targetInformation import *
from chembl_core_model.models.bindingSiteInformation import *
from chembl_core_model.models.experimentalData import *
from chembl_core_model.models.approvedDrugData import *
from chembl_core_model.models.mechanismAnnotation import *


